#! /bin/sh
#
# autodrm.com    19.4.91 / uk    Version of 24.4.95 / uk
#
# This script is scheduled each time we want to check if there is mail for
# autodrm. If there is mail for autodrm, the necessary programs are called.
#
# New version (8.6.99 / uk):
# Allow for "parallel processing", if there is more than one working
# directory. Working directories must be directly below the $HOME directory
# and must begin with the string work (e.g. work1, work2, etc.).
# Parallel processing can be tuned: more working directories means more parallel
# processes. You may disable usage of any working directory by creating a file
# called autodrm.lock in a working directory (but make sure, at least one
# working directory is existing and does not have a lock-file in it).
# And: Be aware that there is a cronjob running every 20 minutes to remove
# such 'unused' lock-files; this is to make sure, that if a machine crashes
# while an autodrm-process is running, AutoDRM will restart to run again
# after the machine is booted!
#
# New version 1.11.2000 :
# Source autodrm.env; useful for extending PATH and for setting variables:
#
#####. $HOME/prog/autodrm.env
#
# New version 2.11.2000 :
# Determine, whether we are called for the fast version or not:
#
if test $# -eq 0 ; then
   fast='false'
else
   fast='true'
   sleep 1
   datestring=`date -u '+%Y%m%d%H%M%S'`
   workdir=`echo $HOME/work$datestring`
   mkdir $workdir
fi
#
# Set mailprogram and path of mail file according to the operating system:
#
mailprogram=`cat $HOME/prog/mail.setup | head -1`
mailfile=`cat $HOME/prog/mail.setup | tail -1`
#
#  If the mail-file exists and has size > 0, there is incoming mail:
#
if test ! -s $mailfile ; then
   exit
else
#
#  Check for a free working directory - if none available, exit and try later:
#
   if test "$fast" = "false" ; then
      workdir="njet"
      for dir in `ls -d $HOME/work*` ; do
         if test ! -r $dir/autodrm.lock ; then
            workdir=$dir
            break
         fi
      done
   fi
   if test "$workdir" = "njet" ; then
      exit      # no free working directory available - exit and try later
   fi
#
   touch $workdir/autodrm.lock
#
   cd $workdir
#
#  Save (append) new mail in a separate file ( $HOME/work/autodrm.mail ):
#
   echo "s * $workdir/autodrm.mail" > $HOME/prog/savemail.cmd_new
   echo "quit" >> $HOME/prog/savemail.cmd_new
   mv $HOME/prog/savemail.cmd_new $HOME/prog/savemail.cmd
   $mailprogram < $HOME/prog/savemail.cmd > /dev/null
#
#  Split the file with the new mails into separate files for each request:
#
   $HOME/bin/splitmailfile.exe
   rm -f $workdir/autodrm.mail
#
#  Now run AutoDRM for each request-mail:
#
   for file in `ls -1 mail.file*` ; do
#
      touch $workdir/autodrm.lock      # touch the lock-file before each run!
      /bin/mv -f $file current.mail
      $HOME/bin/autodrm.exe >> $HOME/log/autodrm.log 2>&1
#
   done
#
#  Remove last request and lock-file:
#
   rm -f $workdir/current.mail
   rm -f $workdir/autodrm.lock
   if "$fast" = "true" ; then
      rm -R $workdir
   fi
#
fi
#
