      program splitmailfile
c
c     Urs Kradolfer, 31.8.94
c
c     Before this program is scheduled, new mail must be saved to a new
c     file. This must be done using the local mail-program and should not
c     be done with a simple move-command!
c
c     Program splitmailfile reads such a saved mail-file and splits it into
c     separate files for each mail.
c
c     For the purpose of AutoDRM, the program checks if there are several
c     request-mails in one e-mail. If so, it creates additional files
c     for each of the request.
c
c     Revision 12.06.95 / uk
c
c     Don *not* search for string BEGIN if mail comes from an autodrm,
c     postmaster or mailer-daemon (if such a mailer returns a user's guide
c     this can cause problems!)
c
c     Also, do not search for string BEGIN if sender is 'Unreplyable!SMTP...'
c     Revision 14.12.95 / uk
c
c     Small changes in subr. FREEFILE because of operating system OSF1
c     Revision 12.1.96 / uk
c
c     Remove leading blanks in lines after the mail-header. This is not really
c     according to GSE2.0 format, but the AutoDRM will just neglect any
c     commands, that are not understood and report them accordingly.
c     --> many people send requests with a blank in column 1 ...!!!
c     --> this problem is now solved.
c     Revision 28.4.96 / uk
c
c     If there are multiple requests within one e-mail (each starting with
c     BEGIN and ending with STOP) then allow this only, if a begin line starts
c     with BEGIN or BEGIN GSE2.0 - otherwise we got problems with spam mails
c     (lines with BEGIN ORDER, etc.). The feature of having several requests
c     within one single mail is anyway seldomly used.
c     Revision 11Nov2001 / uk
c
      implicit none
c
      integer im,it, nheaderlines, irequest, i, trimlen, ntempfiles,
     &        trimbeg
      character*64 mailfile, tempfile
      character beglin*50, creceivedate*20, fromline*180
      character*1024 cinpline, cline, cheaderline(200) ! 200: see also below!!
      logical readingheader, requestmail
c
      mailfile='autodrm.mail'
c
      irequest=0
      nheaderlines=0
      ntempfiles=0
      requestmail=.true.
      call FREEUNIT(im)
      open(im,file=mailfile,status='old')
  1   read(im,'(a)',end=999) cinpline
      cline=cinpline(trimbeg(cinpline):)
      call MAILTEST(cline,creceivedate)   ! is this the start of a new mail?
      if(creceivedate.ne.' ')then
         if(ntempfiles.gt.0) close(it)
         call FREEUNIT(it)
         ntempfiles=ntempfiles+1
         tempfile='mail.file' 
         call FREEFILE(tempfile)
         open(it,file=tempfile,status='unknown')
         irequest=0
         nheaderlines=0
         readingheader=.true.
         requestmail=.true.
      endif
c
c     Check, if mail is coming from somewhere, where no request is expected:
c
      if(cline(1:5).eq.'From:'.and.requestmail)then
         fromline=cline
         call CASEFOLD(fromline)
         if(INDEX(fromline,'MAILER-DAEMON').gt.0)then
            requestmail=.false.
         endif
         if(INDEX(fromline,'AUTODRM').gt.0)then
            requestmail=.false.
         endif
         if(INDEX(fromline,'POSTMASTER').gt.0)then
            requestmail=.false.
         endif
         if(INDEX(fromline,'UNREPLYABLE').gt.0)then
            requestmail=.false.
         endif
      endif
c
c     while reading a non-request mail do not search for BEGIN string:
c     (this is important in case such a mailer returns a user guide or in
c     similar cases!)
c
      if(.not.requestmail) goto 888
c
      beglin=cline(1:50)
      call CASEFOLD(beglin)
      if(nheaderlines.eq.200)then ! stop reading header - it's a bad request!!
         beglin='BEGIN'        ! by doing this, we can write the lines read 
         irequest=irequest+1      ! so far further below!
         readingheader=.false.
         nheaderlines=199         ! we'll miss 1 line but never come here again!
      endif
      if(readingheader)then
         if(beglin.eq.'BEGIN'.or.beglin.eq.'BEGIN GSE2.0')then
            irequest=irequest+1
            readingheader=.false.
         else
            nheaderlines=nheaderlines+1
            cheaderline(nheaderlines)=cline
         endif
      else
         if(beglin.eq.'BEGIN'.or.beglin.eq.'BEGIN GSE2.0')then
            if(ntempfiles.gt.0) close(it)
            call FREEUNIT(it)
            ntempfiles=ntempfiles+1
            tempfile='mail.file' 
            call FREEFILE(tempfile)
            open(it,file=tempfile,status='unknown')
            irequest=irequest+1
            do i=1,nheaderlines
               write(it,'(a)') cheaderline(i)(1:trimlen(cheaderline(i)))
            enddo
         endif
      endif
 888  write(it,'(a)') cline(trimbeg(cline):trimlen(cline))
      goto 1
 999  close(im)
      close(it)
c
cd     write(6,*)'Number of files created:', ntempfiles
c
      end ! of program splitmailfile
c
      subroutine FREEUNIT(iunit)
c
c     get a free fortran file-unit number that is not yet in use
c
      implicit none
c
c     Purpose: Get a free FORTRAN-unit
c
c     Output: iunit  a free (file-)unit number
c
c     Urs Kradolfer, 16.7.90
c
      integer iunit
      logical lopen
c
      do iunit=10,999
         if(iunit.eq.999) stop'FREEUNIT>>> no free unit found!'
         inquire(unit=iunit,opened=lopen)
         if(.not.lopen) RETURN
      enddo
c
      RETURN
c
      end ! of subr. freeunit
c
      subroutine FREEFILE(filename)
c
c     get a unique file name
c
      implicit none
c
c     Purpose: get a unique filename
c
c     Input:  A trial filename
c     Output: A unique filename, starting with the trial filename
c
c     Urs Kradolfer, 16.7.90
c
      integer i, trimbeg,trimlen, i1,i2
      character*(*) filename
      character*100 testfilename, startfilename, string
      logical lexist
c
      i=0
ccc OSF1 does not like this!      if(filename.eq.' '.or.filename.eq.'')then
      if(filename.eq.' ')then
         startfilename='freefile'
      else
         startfilename=filename
      endif
      testfilename=startfilename
 1    inquire(file=testfilename,exist=lexist)
      if(.not.lexist)then
          filename=testfilename
          goto 888 ! RETURN
      else
         i=i+1
         write(string,*) i
         i1=trimbeg(string)
         i2=trimlen(string)
         testfilename=startfilename(1:trimlen(startfilename))//
     &                string(i1:i2)
         goto 1
      endif
c
 888  RETURN
c
      end ! of subr. freefile
c
      subroutine CASEFOLD(cn) ! Urs Kradolfer, 20. Feb. 1990
c
c     put all characters of the inout string into UPPER CASE letters
c
c
c Input : character-string of any length
c Output: same character-string, but all letters are CAPITAL now
c         (other characters are not changed)
c
c   --> this subr. needs the function TRIMLEN !
c
c Call  : call CASEFOLD(charstring)
c
      implicit none
      character cn*(*)
      integer i, trimlen,ilen, ival
      ilen=trimlen(cn)
      do i=1,ilen
         ival=ICHAR(cn(i:i))
         if(ival.ge.97.and.ival.le.122)then
            cn(i:i)=CHAR(ival-32)
         endif
      enddo
      RETURN
      end ! of subroutine casefold
c
      subroutine MAILTEST(line,cdate)
c
c     Urs Kradolfer, 26.9.91
c
c     Input:   a character string (e.g. a line of a mail file)
c     Output:  if the line is the first line of a mail header, the
c              receiving date of the mail is returned, otherwise
c              a blank string is returned
c
c     This routine has been tested on the following operating systems:
c     HP-UX, SunOS, ULTRIX
c
      implicit none
      integer trimlen, i, k, iword, j
      character*(*) line, cdate
      character string*200
      logical newword
c
      cdate=' '
      string=line
      if(string(1:5).ne.'From ') RETURN
      string=string(6:)
      iword=0
      newword=.false.
      k=trimlen(string)
      if(k.lt.20) RETURN
      do i=1,k
         if(string(i:i).eq.' ')then
            newword=.false.
         else
            if(.not.newword)then
               iword=iword+1 
               newword=.true.
               if(iword.eq.5)then
                  j=INDEX(string(i:i+2),':')
                  if(j.le.0) RETURN  ! ULTRIX wants a colon in the 5th word
               endif
               if(iword.eq.8) RETURN  ! ULTRIX accepts only 6 or 7 'words'
            endif
         endif
      enddo
c
999   continue
      if(iword.eq.6.or.iword.eq.7)then
         if(string(k-4:k-4).eq.' '.and.string(k-8:k-8).eq.' ')then
            string=string(:k-8)//string(k-3:k)
            k=trimlen(string)
         endif
         cdate=string(k-19:k)
         string=cdate
         if(cdate(10:10).ne.':')then
            cdate=string(4:16)//'   '//string(17:20)
         endif
      endif
c
      RETURN
      end ! of subr. mailtest
c
      integer function TRIMLEN(t)   ! Urs Kradolfer, June 1986
c
c     Call:    nc=TRIMLEN(char)
c
c          --> nc says, how many characters the input-string has
c              (ignoring trailing blanks!).
c
      implicit none
      character t*(*)
      do 1 trimlen=LEN(t),1,-1
    1    if(t(trimlen:trimlen).ne.' ')RETURN
      trimlen=1
      end ! of integer function trimlen
c
      integer function TRIMBEG(t)   ! Urs Kradolfer, 17.1.90
c
c     Call:    nc=TRIMBEG(char)
c
c          --> nc returns the position of the first characters of 
c              the input-string (ignoring leading blanks!).
c
      implicit none
      character t*(*)
      do 1 trimbeg=1,LEN(t)
    1    if(t(trimbeg:trimbeg).ne.' ')RETURN
      trimbeg=1
      end ! of integer function trimbeg
c
